#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from openstack_dashboard.test.integration_tests.regions import forms
from openstack_dashboard.test.integration_tests.regions import tables
from selenium.webdriver.common import by

from sahara_dashboard.test.integration_tests.pages import basepage
from sahara_dashboard.test.integration_tests.pages import mixins


class CreateMixin(object):
    CREATE_FIELD_MAPPING = (
        ('job_name', 'job_type', 'main_binary', 'job_description'),
        ('lib_binaries',),
    )

    LAUNCH_ON_EXIST_CLUSTER_FIELD_MAPPING = (
        ('job_input', 'job_output', 'cluster'),
        ('adapt_spark_swift', 'datasource_substitute', 'streaming_mapper',
            'streaming_reducer'),
        (),
    )

    @tables.bind_table_action('create job')
    def get_create_form(self, button):
        button.click()
        return forms.TabbedFormRegion(self.driver, self.conf,
                                      field_mappings=self.CREATE_FIELD_MAPPING)

    @tables.bind_row_action('launch-job-existing')
    def get_launch_on_exists_form(self, button, row):
        button.click()
        return forms.TabbedFormRegion(
            self.driver, self.conf,
            field_mappings=self.LAUNCH_ON_EXIST_CLUSTER_FIELD_MAPPING)


class JobtemplatesPage(mixins.DeleteMixin, basepage.BaseDataProcessingPage):

    TABLE_NAME = "job_templates"

    @classmethod
    def get_table_mixins(cls):
        return super(JobtemplatesPage, cls).get_table_mixins() + (CreateMixin,)

    def create(self, name, job_type, binary_name, libs=()):
        form = self.table.get_create_form()
        form.job_name.text = name
        form.job_type.text = job_type
        if binary_name is not None:
            form.main_binary.text = binary_name
        form.switch_to(1)
        for lib in libs:
            form.lib_binaries.text = lib
            form.src_elem.find_element_by_id("add_lib_button").click()
        form.submit()

    def launch_on_exists(self, job_name, input_name, output_name,
                         cluster_name, adapt_swift=True,
                         datasource_substitution=True, configuration=None,
                         parameters=None, arguments=(),
                         mapper=None, reducer=None):
        configuration = configuration or {}
        parameters = parameters or {}
        row = self._get_row_with_name(job_name)
        form = self.table.get_launch_on_exists_form(row)
        if input_name is not None:
            form.job_input.text = input_name
        if output_name is not None:
            form.job_output.text = output_name
        form.cluster.text = cluster_name

        form.switch_to(1)
        if adapt_swift:
            form.adapt_spark_swift.mark()
        else:
            form.adapt_spark_swift.unmark()
        if datasource_substitution:
            form.datasource_substitute.mark()
        else:
            form.datasource_substitute.unmark()

        if mapper is not None:
            form.streaming_mapper.text = mapper
        if reducer is not None:
            form.streaming_reducer.text = reducer

        locator = (by.By.ID, 'configs')
        if form._is_element_visible(*locator):
            config_block = form.src_elem.find_element(*locator)
            add_btn = config_block.find_element_by_link_text('Add')
            for key, value in configuration.items():
                add_btn.click()
                inputs = config_block.find_elements_by_css_selector(
                    'input[type=text]')[-2:]
                inputs[0].send_keys(key)
                inputs[1].send_keys(value)

        locator = (by.By.ID, 'params')
        if form._is_element_visible(*locator):
            params_block = form.src_elem.find_element(*locator)
            add_btn = params_block.find_element_by_link_text('Add')
            for key, value in parameters.items():
                add_btn.click()
                inputs = params_block.find_elements_by_css_selector(
                    'input[type=text]')[-2:]
                inputs[0].send_keys(key)
                inputs[1].send_keys(value)

        locator = (by.By.ID, 'args_array')
        if form._is_element_visible(*locator):
            args_block = form.src_elem.find_element(*locator)
            add_btn = args_block.find_element_by_link_text('Add')
            for value in arguments:
                add_btn.click()
                input_el = args_block.find_elements_by_css_selector(
                    'input[type=text]')[-1]
                input_el.send_keys(value)
        form.submit()



# Copyright 2016 Rec Hat Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from django.utils.translation import ugettext_lazy as _

from horizon import tabs

from sahara_dashboard.content.data_processing.jobs.data_sources \
    import tabs as data_source_tabs
from sahara_dashboard.content.data_processing.jobs.job_binaries \
    import tabs as job_binary_tabs
from sahara_dashboard.content.data_processing.jobs.job_templates \
    import tabs as job_template_tabs
from sahara_dashboard.content.data_processing.jobs.jobs \
    import tabs as job_tabs
from sahara_dashboard.content.data_processing.tabs \
    import PaginationFriendlyTabGroup


class JobTabs(PaginationFriendlyTabGroup):
    slug = "job_tabs"
    tabs = (job_tabs.JobsTab,
            job_template_tabs.JobTemplatesTab,
            data_source_tabs.DataSourcesTab,
            job_binary_tabs.JobBinariesTab,)
    sticky = True


class IndexView(tabs.TabbedTableView):
    tab_group_class = JobTabs
    template_name = "jobs/index.html"
    page_title = _("Jobs")



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from openstack_dashboard.test.integration_tests.regions import forms


class TabbedFormRegion(forms.TabbedFormRegion):

    def __init__(self, driver, conf, field_mappings=None, default_tab=0):
        self.original_field_mappings = field_mappings
        super(TabbedFormRegion, self).__init__(driver, conf,
                                               field_mappings=field_mappings)

    def _set_checkbox_group(self, group_name, values=()):
        for el in self.src_elem.find_elements_by_xpath(
            './/input[@name="{}"]'.format(group_name)
        ):
            elem_id = el.get_attribute('id')
            label = self.src_elem.find_element_by_css_selector(
                'label[for={}]'.format(elem_id))
            if (label.text in values) != el.is_selected():
                label.click()

    def set_values(self, **kwargs):
        for tab_num, fields in enumerate(self.original_field_mappings):
            self.switch_to(tab_num)
            for key in fields:
                value = kwargs.get(key)
                if value is None:
                    continue
                if isinstance(value, (list, tuple)):
                    self._set_checkbox_group(key, value)
                    continue
                field = getattr(self, key)
                if hasattr(field, 'mark'):
                    if value:
                        field.mark()
                    else:
                        field.unmark()
                else:
                    field.text = value



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from horizon import exceptions
from horizon import tabs
from horizon.tabs import base

from sahara_dashboard import utils as u


class SaharaTableTab(tabs.TableTab):
    def get_server_filter_info(self, request, table):
        filter_action = table._meta._filter_action
        if filter_action is None or filter_action.filter_type != 'server':
            return None
        param_name = filter_action.get_param_name()
        filter_string = request.POST.get(param_name)
        filter_string_session = request.session.get(param_name, "")
        changed = (filter_string is not None
                   and filter_string != filter_string_session)
        if filter_string is None:
            filter_string = filter_string_session
        filter_field_param = param_name + '_field'
        filter_field = request.POST.get(filter_field_param)
        filter_field_session = request.session.get(filter_field_param)
        if filter_field is None and filter_field_session is not None:
            filter_field = filter_field_session
        setattr(table.base_actions["filter"], "filter_string", filter_string)
        setattr(table.base_actions["filter"], "filter_field", filter_field)
        filter_info = {
            'action': filter_action,
            'value_param': param_name,
            'value': filter_string,
            'field_param': filter_field_param,
            'field': filter_field,
            'changed': changed
        }
        return filter_info


class PaginationFriendlyTabGroup(tabs.TabGroup):
    def load_tab_data(self):
        """Preload all data that for the tabs that will be displayed."""
        request_without_marker = u.delete_pagination_params_from_request(
            self.request, save_limit=True)
        for tab in self._tabs.values():
            current_tab_id = tab.slug

            tab_request = self.request.GET.get('tab')
            request_tab_id = None

            if tab_request:
                request_tab_id = tab_request.split(base.SEPARATOR)[1]
            if request_tab_id and current_tab_id != request_tab_id:
                try:
                    tab.request = request_without_marker
                    tab._data = tab.get_context_data(request_without_marker)
                except Exception:
                    tab._data = False
                    exceptions.handle(request_without_marker)

            if tab.load and not tab.data_loaded:
                try:
                    tab._data = tab.get_context_data(self.request)
                except Exception:
                    tab._data = False
                    exceptions.handle(self.request)
